function [] = mx_derive_N2_ref(moving_window,dzN2ref)

% mx_derive_N2_ref               Derive the reference buoyancy frequency N2
%==========================================================================
%
% USAGE:
%  [] = mx_derive_N2_ref(moving_window,dzN2ref)
%
% DESCRIPTION:
% Derives a reference N2 (N2_ref) profile based on a x (moving_window 
% eg 20) profiles moving average of each individual N2 profiles. First x/2 
% N2_ref are based on profiles 1-x and last x+1 profiles are mean of 
% profile end-(x-1):end. Since profiles have different length, interps the 
% bottom of the N2_ref profile from a certain depth onwards based on how 
% many values go into each point.
%         
% INPUT:
%   dzN2ref         = bin depth for N2 computation
%   moving_window   = for eg 40
%
% OUTPUT:                     
%   N2_ref          =  Brunt-Vaisala Frequency squared from Polzin code   [ 1/s^2 ]
%
% AUTHOR: 
%   Amelie MEYER based on Kurt Polzin original code.
%
% VERSION NUMBER: 1.0 (24th June, 2014)
%==========================================================================
display('Derive the reference buoyancy frequency (N2_ref)...');

warning('off','MATLAB:polyfit:RepeatedPointsOrRescale')

load initial_data_gridded.mat
n=moving_window;

%% DERIVE N2 PROFILE
s=initial_data_gridded.SA;
t=initial_data_gridded.CT;
p=initial_data_gridded.Pctd;
N2=initial_data_gridded.N2;
lat=initial_data_gridded.lat;
PID=initial_data_gridded.profile_number;
% Looks how many profiles there are:
profiles=length(initial_data_gridded.fltid);
% Creates a new empty variable N2_ref:
N2_ref(1:length(s(:,1)),profiles)=NaN; 
% Derive N2:
n2(1:length(s(:,1)),profiles)=NaN;
i_min=firstgood2(s);    % finds 1st non NaN value in S_ref
i_max=lastgood2(s);     % finds last non NaN value in S_ref
n2(i_min:i_max,:)=nsq_modbs(s(i_min:i_max,:),t(i_min:i_max,:),p(i_min:i_max,:),dzN2ref,lat);

%%  DERIVE RUNNING MEAN N2 PROFILE (N2_REF)
for i=1:profiles
    % Find indexes to make mean N2 profile
    a=i-(n/2);      % index of 1st profile going in the mean
    b=i+(n/2)-1;    % index of last profile going in the mean 

    % For 1st n/2+1 profiles, uses the 1st n profiles:
    if i<(n/2)+1;   
        a=1; b=n;
    end
    if i>(profiles-n/2)
        a=profiles-n+1;
        b=profiles;
    end
    % Calculate n2_ref based on mean of all N2 6db chosen
    n2_ref1(:,i)=nanmean(n2(:,a:b),2); 

    % Check how far down the watercolumn all of the profiles that
    % have data going into the n2_ref value go:
    for ii=1:length(s(:,i));
        t=sum(isfinite(s(end-(ii-1),:)));
        if t>n  
            % ip_max is index of max depth at which all of profiles still go
            ip_max=length(s(:,1))-(ii-1);        
            break
        end
    end; clear ii t

    % Replace those bottom values with NaN where less than 90% profiles had data
    n2_ref2=n2_ref1;
    n2_ref2(ip_max+1:end,i)=NaN;
    
    % Interp the bottom of the mean profile that was cut off:
    for iii=1:length(s(:,1));
        t=sum(isfinite(s(end-(iii-1),:)));
        if t>0;
            % works out whats the deepest data point out of all profiles 
            % which is going to be the deepest N2ref we calculate
            i_mmax=length(s(:,1))-(iii-1); 
            break
        end
    end; clear iii t

    fit=polyfit(p(600:ip_max,1),n2_ref2(600:ip_max,i),3);
    fit2(1:length(n2_ref1))=NaN; % create a NaN variable
    fit2(600:i_mmax)=polyval(fit,p(600:i_mmax,1));
    n2_ref=n2_ref2;
    % Add the interpolation bit to the bottom of the profile:
    n2_ref(ip_max+1:i_mmax,i)=fit2(ip_max+1:i_mmax);

    % Finds last non NaN value in N2 (local profile):
    ii_max=max(find(isnan(N2(:,i))==0));       
    % Cut off N2_ref profile underneath ii_max:
    n2_ref(ii_max+1:end,i)=NaN;                     
end;

%% NEW STRUCTURE
mixing_data.N2_ref(1:length(s(:,1)),1:profiles)=NaN;
% Single values
mixing_data.fltid=initial_data_gridded.fltid;
mixing_data.profile_number=initial_data_gridded.profile_number;
mixing_data.lon=initial_data_gridded.lon;
mixing_data.lat=initial_data_gridded.lat;
mixing_data.surface_mlt=initial_data_gridded.surface_mlt;
mixing_data.MLD=initial_data_gridded.MLD;
% Many profiles
mixing_data.Pctd=initial_data_gridded.Pctd;
mixing_data.SA=initial_data_gridded.SA;
mixing_data.CT=initial_data_gridded.CT;
mixing_data.sigma_0=initial_data_gridded.sigma_0;
mixing_data.N2=initial_data_gridded.N2;
mixing_data.N2_ref=n2_ref;
mixing_data.Pef=initial_data_gridded.Pef;
mixing_data.fallrate=initial_data_gridded.fallrate;
mixing_data.U=initial_data_gridded.U;
mixing_data.V=initial_data_gridded.V;
mixing_data.speed=initial_data_gridded.speed;

save('mixing_data.mat','-regexp','^mixing_data','^flt');