%==========================================================================
% mx_derive_mixing_shear
%
% DESCRIPTION:
% Computes the dissipation rate (epsilon) and diapycnal 
% turbulent eddy diffusivity (kappa) using Henyey, Wright and Flatte 
% model fine scale parameterization of turbulent dissipation using shear 
% information only. This is computed in depth bins starting at the surface
% then down.  Implication:
% the bottom-most depth bin i snot full depth and the spectral 
% estimate in the bottom-most depth bin is maybe compromised.
%       
% INPUT:
%   Vertical profile of pressure and vertical shear (complex).
%
% OUTPUT:                     
%   Shear spectra for various depth segments of the profile and
% estimates of dissipation and diffusivity for each depth segment.   
%
% AUTHOR:		Alberto NAVEIRA GARABATO
%				
% Revisited by:	Stephanie WATERMAN 2010
%				Amelie MEYER June 2014
%
% VERSION NUMBER: 1.0 (25th June, 2014)
%
% REFERENCES:
% Henyey, F. S., Wright, J., and Flatte, S.M., 1986: Energy and action 
% flow through the internal wave field: an eikonal approach. 
% Journal of Geophysical Research, 91, 8487-8496
%
%==========================================================================

load mixing_data_gridded.mat
profiles=length(mixing_data_gridded.fltid);

%% FOR EACH PROFILE
for i=1:profiles;
    % List of indices at which we have shear data
    ii=find(isnan(mixing_data_gridded.shear(:,i))==0);
    if isempty(ii)==1;
        % If there is no shear data in that pofile, add empty (NaN) 
        % variables to structure:
        li=mixing_data_gridded.P(:,i);
        mixing_data_gridded.epsilon_shear(1:li,i)=NaN;
        mixing_data_gridded.Kz_shear(1:li,i)=NaN;
        mixing_data_gridded.P_m_shear(1:li,i)=NaN'';
        mixing_data_gridded.shear_variance(1:li,i)=NaN'';
        mixing_data_gridded.CW_S_variance(1:li,i)=NaN;
        mixing_data_gridded.CCW_S_variance(1:li,i)=NaN;
        mixing_data_gridded.Mc_shear(1:li,i)=NaN;

    else % if there are shear data, derive variables:
        i_min=min(ii);                          % smallest indice
        p_min=mixing_data_gridded.P(i_min,i);   % corresponding depth

        % Find maximum pressure with velocity measurement:
        i_max=max(ii);
        p_max=mixing_data_gridded.P(i_max,i); % corresponding depth

        % Define dp centre scale (centre points of depth segments) 
        % top-down for this station; calculation will result in
        % epsilon/kappa values at dp centre:
        dp_centre_here=[(p_min+dp_overlap):dp_resolution:p_max-dp_overlap];

        % Make some variables to store the spectra from each depth bin 
        % in so you can go back and look at them later:
        spectra=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_mod=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_CCW=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_CCW_mod=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_CW=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_CW_mod=ones(length(dp_centre_here),fftpt+1)*NaN;
        spectra_GM=ones(length(dp_centre_here),fftpt+1)*NaN;
        freq_scale=ones(length(dp_centre_here),fftpt+1)*NaN;
        kzax_scale=ones(length(dp_centre_here),fftpt+1)*NaN;


        %% FOR EACH DEPTH BIN
        for idp=1:length(dp_centre_here)

            % Select pressure interval for this depth bin
            % Upper bound of depth segment:
            imin=max(1,(min(find(mixing_data_gridded.P(:,i)>=(dp_centre_here(idp)-dp_overlap))))); 
            pressure_min=mixing_data_gridded.P(imin,i);
            % Lower bound of depth segment
            imax=min((max(find(mixing_data_gridded.P(:,i)<(dp_centre_here(idp)+dp_overlap)))),size(mixing_data_gridded.P(:,1),1)); 
            pressure_max=mixing_data_gridded.P(imax,i);

            %% NORMALISE BY N2 
            % Normalise vertical shear segment by mean N
            % Calculate mean N over pressure interval:
            [yy imin_CTD]=min(abs(mixing_data_gridded.P(:,i)-pressure_min));
            [yy imax_CTD]=min(abs(mixing_data_gridded.P(:,i)-pressure_max));

            if imax_CTD<length(mixing_data_gridded.P(:,i))
                imax_CTD=imax_CTD+1;
            end

            pressure_min_CTD=mixing_data_gridded.P(imin_CTD,i);
            pressure_max_CTD=mixing_data_gridded.P(imax_CTD,i);

            % Compute mean N^2 for this depth bin:
            n_mean=sqrt(nanmean(mixing_data_gridded.N2(imin_CTD:imax_CTD,i)));
            n_ref_mean=sqrt(nanmean(mixing_data_gridded.N2_ref(imin_CTD:imax_CTD,i)));

            % Normalize shearfor selected segment
            shear=NaN*ones(length(imin:imax));
            shear=mixing_data_gridded.shear(imin:imax,i)./n_ref_mean; 
            
            %% DERIVE THE SHEAR SPECTRA
            % As well as the cospectra and rotary spectra for segment

            shearttemp = shear(~isnan(shear(:)));
            N = length(shearttemp);
            ratio = N/length(shear);
            x=shearttemp;

            if isempty(x)==0
                p_grid(idp)=dp_centre_here(idp);
                pressure_min_segment_CTD(idp)=pressure_min_CTD;
                pressure_max_segment_CTD(idp)=pressure_max_CTD;
                n_mean_segment(idp)=n_mean;
                n_ref_mean_segment(idp)=n_ref_mean;
                pressure_min_segment(idp)=pressure_min;
                pressure_max_segment(idp)=pressure_max;
                % Number of points in spectra calculation not to be 
                % confused with the bouyancy frequency:
                N_segment(idp)=N; 
                ratio_segment(idp)=ratio;
                % dz2 assumes depthbin uniform over depthrange of transform
                dz2 = sw_dpth(nanmean(diff(mixing_data_gridded.P(imin:imax,i))),mixing_data_gridded.lat(i)); 
                Fs = 1/dz2;
                x=real(shearttemp);
                y=imag(shearttemp);
                % We use Tycho2_cospectra.m to derive power and variance 
                % spectrum. Kzax, one of the output is  the variance of 
                % the shear.
                eval(parameters.cospectral_method) 

                %% CORRECTIONS
                % Sampled data to improve the high wavenumber part of the
                % spectra. See Polzin et al. (2002) for details.

                % 1) first-differencing:
                if parameters.switch_fd==1
                    Tfd = sinc(kzax*dzfd/(2*pi)).^2;
                else
                    Tfd = 1;
                end

                % 2) Voltmeter correction
                % kzax=m (wave number), W=fall rate (ms-1), 50s represents
                % 50 points of 1s data (fitting interval)
                W_mean=nanmean(mixing_data_gridded.fallrate(imin_CTD:imax_CTD,i));
                Tvolt=1./[sinc(kzax*50*W_mean/(2*pi))];

                % Calculate model transfer function
                Tmod=Tvolt.*Tfd;

                % Correct the power spectral density
                spec_mod=spec./Tmod';
                spec_CCW_mod=spec_CCW./Tmod';
                spec_CW_mod=spec_CW./Tmod';


                %% CRITICAL WAVENUMBER
                % Using the info we have of local N2, we work out critical
                % wavenumber mc which gives us better estimate of the
                % minimum vertical wavelength of integration.

                shearvariance=kzax(2)*cumsum(spec_mod);
                [nnn,index]=min(abs(shearvariance-0.7));

                if shearvariance(end)>0.7
                    index=find(shearvariance-0.7>0,1);
                    if index==1
                        index=round(length(shearvariance)/2);
                        criticalm(idp)=kzax(index)*(0.7/shearvariance(index))/(2*pi);
                        minwavelength(idp)=1/criticalm(idp);
                    else
                    fraction=(0.7-shearvariance(index-1))/(shearvariance(index)-shearvariance(index-1));
                    criticalm(idp)=(kzax(index)+kzax(2)*(0.5+fraction))/(2*pi);
                    minwavelength(idp)=1/criticalm(idp);
                    end
                else index=round(length(shearvariance)/2);
                    criticalm(idp)=kzax(index)*(0.7/shearvariance(index))/(2*pi);
                    minwavelength(idp)=1/criticalm(idp);
                end

                ind(idp)=index;


                %% LINEAR INTERPOLATION
                % Linearly interpolate onto finer frequency axis makes
                % integrating between certain wavelengths more accurate.
                lambda_max=1/freq(2);
                lambda_min=1/freq(end);
                wavelengths=lambda_min:1:lambda_max;
                freq2=1./wavelengths;

                spec2 = interp1(freq,spec,freq2);
                spec_mod2 = interp1(freq,spec_mod,freq2);
                spec_CCW2 = interp1(freq,spec_CCW,freq2);
                spec_CCW_mod2 = interp1(freq,spec_CCW_mod,freq2);
                spec_CW2 = interp1(freq,spec,freq2);
                spec_CW_mod2 = interp1(freq,spec_CW_mod,freq2);
                kzax2 = interp1(freq,kzax,freq2);

                spec=spec2;
                spec_mod=spec_mod2;
                spec_CCW=spec_CCW2;
                spec_CCW_mod=spec_CCW_mod2;
                spec_CW=spec_CW2;
                spec_CW_mod=spec_CW_mod2;
                kzax=kzax2;
                freq=freq2;

                % Store spectra for this depth bin
                for ii=1:length(spec)
                    spectra(idp,ii)=spec(ii);
                    spectra_mod(idp,ii)=spec_mod(ii);
                    spectra_mod2(idp,ii)=spec_mod2(ii); % new scale spectra

                    spectra_CCW(idp,ii)=spec_CCW(ii);
                    spectra_CCW_mod(idp,ii)=spec_CCW_mod(ii);
                    spectra_CCW_mod2(idp,ii)=spec_CCW_mod2(ii);

                    spectra_CW(idp,ii)=spec_CW(ii);
                    spectra_CW_mod(idp,ii)=spec_CW_mod(ii);
                    spectra_CW_mod2(idp,ii)=spec_CW_mod2(ii);                    

                    freq_scale(idp,ii)=freq(ii);
                    kzax_scale(idp,ii)=kzax(ii);
                    freq_scale2(idp,ii)=freq2(ii);
                    kzax_scale2(idp,ii)=kzax2(ii);                      
                end


                %% INTEGRATED SHEAR VARIANCE 
                % Consider the integrated shear variance between two
                % wavelengths of interest. It is the important part of the
                % fine structure estimate of epsilon/kappa.

                % Define lzmax and calculate lzmin, the minimum vertical
                % wavelength of integration:
                lzmin(idp)=minwavelength(idp);
                lzmax(idp)=(2*pi/min(kzax))-1;
                % Could also chose to use the fixed values:
                    % lzmin(idp)=lzmin_fixed;
                    % lzmax(idp)=lzmax_fixed;

                % Integrate power spectral density between lzmin and lzmax:
                imax_int=min(find(kzax<(2*pi/lzmax(idp))));
                imin_int=max(find(kzax>(2*pi/lzmin(idp))));

                if isempty(imin_int)==0 & isempty(imax_int)==0
                    [rows columns]=size(kzax);
                    if rows~=1
                        kzax=kzax';
                    end
                    vector1=cat(2,kzax(imin_int),kzax(imin_int:imax_int),kzax(imax_int),kzax(imin_int));

                    % Shear variance:
                    temp=spec_mod(imin_int:imax_int);
                    [rows columns]=size(spec_mod);
                    if rows~=1
                        temp=temp';
                    end
                    vector2=cat(2,0,temp,0,0);
                    variance_int=polyarea(vector1,vector2);

                    % Variance CCW shear:
                    temp=spec_CCW_mod(imin_int:imax_int);
                    [rows columns]=size(spec_mod);
                    if rows~=1
                        temp=temp';
                    end
                    vector2=cat(2,0,temp,0,0);
                    variance_int_CCW=polyarea(vector1,vector2);

                    % Variance CW shear:
                    temp=spec_CW_mod(imin_int:imax_int);
                    [rows columns]=size(spec_mod);
                    if rows~=1
                        temp=temp';
                    end
                    vector2=cat(2,0,temp,0,0);
                    variance_int_CW=polyarea(vector1,vector2);
                    variance_integral(idp)=variance_int;
                    variance_integral_CCW(idp)=variance_int_CCW;
                    variance_integral_CW(idp)=variance_int_CW;

                    kmin_integral(idp)=kzax(imin_int);
                    kmax_integral(idp)=kzax(imax_int);
                    no_pts_integral(idp)=imax_int-imin_int+1;


                    %% DERIVE DISSIPATION AND DIFFUSIVITY 
                    % For normalization purposes we compute power spectral 
                    % density of normalized vertical shear for the GM76 model
                    N2mean = n_ref_mean^2;

                    % Power spectral density of vertical shear normalised
                    % by N for the GM76 model:
                    betastar=pi*parameters.jstar/parameters.b*sqrt(N2mean)/parameters.N0;
                    
                    % Power spectral density of horizontal velocity:
                    phi_u = 3*parameters.E*parameters.b^3*parameters.N0^2/(2*parameters.jstar*pi) ./(1+kzax/betastar).^2; 
                    % Power spectral density of vertical shear normalised 
                    phi_sn = kzax.^2.*phi_u/N2mean; 

                    for ii=1:length(phi_sn)
                        spectra_GM(idp,ii)=phi_sn(ii);
                    end

                    % Integrate GM76 power spectral density up to cutoff 
                    % wavelength
                    if isempty(imin_int)==0 & isempty(imax_int)==0

                        vector1=cat(2,kzax(imin_int),kzax(imin_int:imax_int),kzax(imax_int),kzax(imin_int));

                        temp=phi_sn(imin_int:imax_int);
                        [rows columns]=size(phi_sn);
                        if rows~=1
                            temp=temp';
                        end
                        vector2=cat(2,0,temp,0,0);

                        GM_variance_integral(idp)=polyarea(vector1,vector2);

                    else
                        GM_variance_integral(idp)=NaN;
                    end

                    % Compute dissipation rate (epsilon) from
                    % the integrated strain variance:
                    epsilon(idp)=parameters.epsilon0*N2mean/parameters.N0^2*variance_integral(idp)^2/GM_variance_integral(idp)^2; % parameterization uncertain to within a factor of 2

                    % Modify esimate of epsilon by a few factors (see 
                    % Kurt's papers for details); First a correction factor
                    % that depends on latitude. Second a correction factor 
                    % that depends on the shear to strain ratio.
                    fmean=abs(sw_f(mixing_data_gridded.lat(i)));
                    extra1 = (fmean/parameters.f0) * acosh(sqrt(N2mean)/fmean)/acosh(parameters.N0/parameters.f0);
                    epsilon(idp) = epsilon(idp)*extra1;
                    extra2=(3*(parameters.R+1))./(2*sqrt(2).*parameters.R.*(abs(parameters.R-1)).^(1/2));
                    
                    epsilon(idp)=epsilon(idp)*extra2;

                    % Finally from epsilon calculate diapycnal turbulent 
                    % eddy diffusivity assuming the Osborn relation:
                    kappa(idp)=parameters.gamma*epsilon(idp)/N2mean;

                    % Add variables to float structure:
                    mixing_data_gridded.epsilon_shear(idp,i)=epsilon(idp);
                    mixing_data_gridded.Kz_shear(idp,i)=kappa(idp);
                    mixing_data_gridded.P_m_shear(idp,i)=p_grid(idp);
                    mixing_data_gridded.shear_variance(idp,i)=variance_integral(idp);
                    mixing_data_gridded.CW_S_variance(idp,i)=variance_integral_CW(idp);
                    mixing_data_gridded.CCW_S_variance(idp,i)=variance_integral_CCW(idp);
                    mixing_data_gridded.Mc_shear(idp,i)=criticalm(idp);

                end
            end   
        end
    end
end

save('mixing_data_gridded.mat','-regexp','^mixing_data_gridded','^flt');