%==========================================================================
% mx_derive_mixing_shearstrain
%
% DESCRIPTION:
% Combines estimates of finestructure epsilon from shear 
% and strain by computing the correction factor dependent on the observed 
% (as opposed to assumed) shear to strain ratio.
% Here uses the "straight-up" shear to strain ratio and assumes that the
% shear variance integral and strain variance integral have been calculated
% using the same wavelength range of integration.
%         
% INPUT:
%
% OUTPUT:                     
%   N2_ref    	=  Brunt-Vaisala Frequency squared from Polzin code   [ 1/s^2 ]
%
% AUTHOR:		Alberto NAVEIRA GARABATO
%				
% Revisited by:	Stephanie WATERMAN 2010
%				Amelie MEYER June 2014
%
% VERSION NUMBER: 1.0 (25th June, 2014)
%
%==========================================================================

profiles=length(mixing_data_gridded.fltid);

for i=1:profiles;                                       % For each profile
    common_p_scale=1:3:1648;
    N2=mixing_data_gridded.N2(:,i);

    %% SHEAR VARIANCE:
    shear_variance=ones(length(common_p_scale),1)*NaN;
    epsilon_shear=ones(length(common_p_scale),1)*NaN;

    % Check if there is any shear data for that profile...
    check=find(isnan(mixing_data_gridded.Kz_shear)==0);
    if isempty(check)==1; 
        % There was no shear data then add empty (NaN) variables 
        li=length(mixing_data_gridded.P(:,i));
        mixing_data_gridded.epsilon_SS(1:li,i)=NaN;
        mixing_data_gridded.Kz_SS(1:li,i)=NaN;
        mixing_data_gridded.P_m_SS(1:li,i)=NaN;
        mixing_data_gridded.Rw(1:li,i)=NaN;

    % If there are shear data then we derive variables:
    else
        for idp=1:length(mixing_data_gridded.P_m_shear(:,i));
            clear ii jj
            ii=find(common_p_scale==mixing_data_gridded.P_m_shear(idp,i)-(dp_resolution/2));
            jj=find(common_p_scale==mixing_data_gridded.P_m_shear(idp,i)+(dp_resolution/2));
            for idp2=ii:jj
                % Copy variables from mixing_data
                shear_variance(idp2)=mixing_data_gridded.shear_variance(idp,i);
                epsilon_shear(idp2)=mixing_data_gridded.epsilon_shear(idp,i);
            end
        end

        %% STRAIN
        % Pre-allocate some variables
        strain_variance=ones(length(common_p_scale),1)*NaN;
        epsilon_strain=ones(length(common_p_scale),1)*NaN;
        N2b=ones(length(common_p_scale),1)*NaN;
        mixing_data_gridded.P_m_shear(isnan(mixing_data_gridded.P_m_shear))=0;
        % Works out hom many d index apart shear and strain are:
        d=mixing_data_gridded.P_m_shear(1,i)-mixing_data_gridded.P_m_strain(1,i)/3; 
        if d>3
            d=0;
        end
        if d<0
            d=0;
        end
        
        for idp=1:length(mixing_data_gridded.P_m_strain(:,i));
            clear ii2 jj2
            ii2=find(common_p_scale==mixing_data_gridded.P_m_strain(idp,i)-(dp_resolution/2));
            jj2=find(common_p_scale==mixing_data_gridded.P_m_strain(idp,i)+(dp_resolution/2));
            % Moving all epsi strain by 1 index down to match epsi 
            % shear scale:
            for idp2=ii2:jj2      
                % Copy variables from mixing_data
                strain_variance(idp2+d)=mixing_data_gridded.strain_variance(idp,i);
                epsilon_strain(idp2+d)=mixing_data_gridded.epsilon_strain(idp,i);
                N2b(idp2+d)=mixing_data_gridded.N2b(idp,i);
            end
        end

        %% FIND POINT OF MAX N2 IN WATER COLUMN
        [crap ind]=max(N2);
        depth(i)=common_p_scale(ind);
        treshhold(i)=depth(i)+dp_overlap;
        [crap index_treshhold(i)]=min(abs(treshhold(i)-common_p_scale));

        %% DERIVE SHEAR STRAIN RATIO
        Rw=shear_variance./strain_variance;
        % Replace Rw values in surface waters with a mean Rw
        Rw(1:index_treshhold(i))=5; % 5 is the mean Rw for the data set

        % Apply extra factor multiplying epsilon depending 
        % on shear to strain ratio using actual shear to strain ratio:
        h=real((3*(Rw+1))./(2*sqrt(2).*Rw.*((Rw-1)).^(1/2)));

        % Remove constant Rw applied:
        epsilon=epsilon_shear;
        extra2=(3*(parameters.R+1))./(2*sqrt(2).*parameters.R.*((parameters.R-1)).^(1/2));
        epsilon2=epsilon/extra2;

        % Apply variable Rw:
        epsilon_shear_and_strain=epsilon2.*h;

        % Finally from epsilon calculate diapycnal diffusivity
        % (kappa) assuming the Osborn relation:
        kappa=parameters.gamma*epsilon_shear_and_strain./N2b;

        % Add variables to float structure: 
        mixing_data_gridded.epsilon(:,i)=epsilon_shear_and_strain;
        mixing_data_gridded.Kz(:,i)=kappa;
        mixing_data_gridded.P_m(:,i)=common_p_scale;
        mixing_data_gridded.Rw(:,i)=Rw;
    end
end

% Remove a variable from the float structure
mixing_data_gridded=rmfield(mixing_data_gridded,'N2b');

eval(['save(''mixing_data_gridded_' run '.mat'',''-regexp'',''^mixing_data_gridded'',''^flt'')']);