%==========================================================================
% mx_derive_mixing_strain
%
% DESCRIPTION:
% Computes the dissipation rate (epsilon) and diapycnal 
% turbulent eddy diffusivity (kappa) using Henyey, Wright and Flatte 
% model fine scale parameterization of turbulent dissipation using strain 
% information only. This is computed in depth bins starting at the surface
% then down.  Implication:
% the bottom-most depth bin i snot full depth and the spectral 
% estimate in the bottom-most depth bin is maybe compromised.
%       
% INPUT:
%   Vertical profile of pressure, N^2, N^2 reference and strain = 
% (N^2-N^2_ref)/N^2_ref.
%
% OUTPUT:                     
%   Strain spectra for various depth segments of the profile and
% estimates of dissipation and diffusivity for each depth segment.   
%
% AUTHOR:		Alberto NAVEIRA GARABATO
%				
% Revisited by:	Stephanie WATERMAN 2010
%				Amelie MEYER June 2014
%
% VERSION NUMBER: 1.0 (26th June, 2014)
%
% REFERENCES:
% Henyey, F. S., Wright, J., and Flatte, S.M., 1986: Energy and action 
% flow through the internal wave field: an eikonal approach. 
% Journal of Geophysical Research, 91, 8487-8496
%
%==========================================================================

load mixing_data_gridded.mat
profiles=length(mixing_data_gridded.fltid);

%% FOR EACH PROFILE
for i=1:profiles;
    % Find minimum pressure of this profile to use as starting point of 
    % first depth segment
    % List of indices at which we have strain:
    ii=find(isnan(mixing_data_gridded.strain(:,i))==0);       
    i_min=min(ii);                          % smallest indice
    i_max=max(ii);                          % smallest indice
    p_min=mixing_data_gridded.P(i_min,i); 	% coprresponding depth
    p_max=mixing_data_gridded.P(i_max,i);  	% coprresponding depth

    % Define dp centre scale (centre points of depth segments) top-down 
    % for this station; calculation will result in epsilon/kappa values at
    % dp centre:
    dp_centre_here=[(p_min+dp_overlap):dp_resolution:6000];

    % Make some variables to store the spectra from each depth bin in so 
    % you can go back and look at them later:
    spectra=ones(length(dp_centre_here),fftpt+1)*NaN;
    spectra_mod=ones(length(dp_centre_here),fftpt+1)*NaN;   % GM corrected
    spectra_GM=ones(length(dp_centre_here),fftpt+1)*NaN;    % GM corrected
    freq_scale=ones(length(dp_centre_here),fftpt+1)*NaN;
    kzax_scale=ones(length(dp_centre_here),fftpt+1)*NaN;


%% FOR EACH DEPTH BIN
for idp=1:length(dp_centre_here)

    % Select pressure interval for this depth bin
    % Upper bound of depth segment:
    imin=max(1,(min(find(mixing_data_gridded.P(:,i)>=(dp_centre_here(idp)-dp_overlap))))); 
    % Lower bound of depth segment:
    imax=min((max(find(mixing_data_gridded.P(:,i)<(dp_centre_here(idp)+dp_overlap)))),size(mixing_data_gridded.P(:,i),1)); 

    pressure_min=mixing_data_gridded.P(imin,i);
    pressure_max=mixing_data_gridded.P(imax,i);

    % Compute mean N^2 for this depth bin:
    n_mean=sqrt(nanmean(mixing_data_gridded.N2(imin:imax,i)));
    n_ref_mean=sqrt(nanmean(mixing_data_gridded.N2_ref(imin:imax,i)));

    %% DERIVE THE STRAIN SPECTRA 
    % Isolate desired depth range and calculate vertical strain spectrum
    strain=mixing_data_gridded.strain(imin:imax,i);

    straintemp = strain(~isnan(strain(:)));
    N = length(straintemp);
    ratio=N/length(strain);

    if length(straintemp)>1
        p_grid(idp)=dp_centre_here(idp);
        pressure_min_segment(idp)=pressure_min;
        pressure_max_segment(idp)=pressure_max;
        n_mean_segment(idp)=n_mean;
        n_ref_mean_segment(idp)=n_ref_mean;
        N_segment(idp)=N;
        ratio_segment(idp)=ratio;

        dz =sw_dpth(nanmean(diff(mixing_data_gridded.P(imin:imax,i))),mixing_data_gridded.lat(i)); % assumes depth bin is uniform over depth range of transform
        Fs = 1/dz;

        x=straintemp;
        eval(parameters.spectral_method)   % Method to derive spectrum

        %% SPECTRAL CORRECTIONS
        % First-differencing
        if parameters.switch_fd==1
            Tfd = sinc(kzax*dzfd/(2*pi)).^2;
        else
            Tfd = 1;
        end

        % Calculate model transfer function
        Tmod = Tfd;

        % Calculate corrected power spectral density
        spec_mod=spec./Tmod';

        %% CRITICAL WAVENUMBER
        % Using the info we have of local N2, we work out critical
        % wavenumber (mc) which gives us better estimate of the
        % minimum vertical wavelength of integration:

        shearvariance=kzax(2)*cumsum(spec);
        [nnn,index]=min(abs(shearvariance-0.7));
        ind(idp)=index;
        criticalm(idp)=kzax(index);
        minwavelength(idp)=1/criticalm(idp)*2*pi;
        if minwavelength(idp)<12;
            minwavelength(idp)=12;
        end

        %% LINEAR INTERPOLATION
        % Linearly interpolate onto finer frequency axis (makes integrating
        % between certain wavelengths more accurate)

        lambda_max=1/freq(2);
        lambda_min=1/freq(end);
        wavelengths=lambda_min:1:lambda_max;
        freq2=1./wavelengths;

        spec2 = interp1(freq,spec,freq2);
        spec_mod2 = interp1(freq,spec_mod,freq2);
        kzax2 = interp1(freq,kzax,freq2);

        % Store spectra for this depth bin so you can look at it later:
        for ii=1:length(spec2)
            spectra(idp,ii)=spec2(ii);
            spectra_mod(idp,ii)=spec_mod2(ii);
            freq_scale(idp,ii)=freq2(ii);
            kzax_scale(idp,ii)=kzax2(ii);
        end

        spec=spec2;
        spec_mod=spec_mod2;
        kzax=kzax2;
        freq=freq2;

        %% INTEGRATED SHEAR VARIANCE 
        % Consider the integrated strain variance between the two
        % wavelengths of interest - this is the meat in the fine structure
        % estimate of mixing

        % Define lzmax and lzmin, the range of vertical
        % wavelengths of integration:
        lzmin(idp)=minwavelength(idp);
        lzmax(idp)=(2*pi/min(kzax))-1;
        % Or could have used the set wavelengths of integration:
            % lzmin(idp)=lzmin_fixed;
            % lzmax(idp)=lzmax_fixed;

        % Integrate power spectral density between lzmin and lzmax:
        imax_int=min(find(kzax<(2*pi/lzmax(idp))));
        imin_int=max(find(kzax>(2*pi/lzmin(idp))));

        if isempty(imin_int)==0 & isempty(imax_int)==0

            [rows columns]=size(kzax);
            if rows~=1
                kzax=kzax';
            end
            vector1=cat(2,kzax(imin_int),kzax(imin_int:imax_int),kzax(imax_int),kzax(imin_int));

            [rows columns]=size(spec_mod);
            if rows~=1
                spec_mod=spec_mod';
            end
            
            vector2=cat(2,0,spec_mod(imin_int:imax_int),0,0);

            % Integrated variance:
            variance_int=polyarea(vector1,vector2); 
            variance_integral(idp)=variance_int;

            kmin_integral(idp)=kzax(imin_int);
            kmax_integral(idp)=kzax(imax_int);
            no_pts_integral(idp)=imax_int-imin_int+1;

            %% DERIVE DISSIPATION AND DIFFUSIVITY 
            % For normalization purposes compute power spectral density of
            % vertical strain for the GM76 model

            N2mean = n_mean.^2;
            N2refmean = n_ref_mean.^2;

            fmean=abs(sw_f(mixing_data_gridded.lat(i)));

            betastar=pi*parameters.jstar/parameters.b*sqrt(N2refmean)/parameters.N0;

            % Power spectral density of vertical displacement:
            phi_zeta = parameters.E*parameters.b^3*parameters.N0^2/(2*parameters.jstar*pi*N2refmean) ./(1+kzax/betastar).^2;             
            % Power spectral density of vertical strain:
            phi_eta = kzax.^2.*phi_zeta; 

            for ii=1:length(phi_eta)
                spectra_GM(idp,ii)=phi_eta(ii);
            end

            %% Integrate GM76 power spectral density
            if isempty(imin_int)==0 & isempty(imax_int)==0
                vector1=cat(2,kzax(imin_int),kzax(imin_int:imax_int),kzax(imax_int),kzax(imin_int));
                [rows columns]=size(phi_eta);
                
                if rows~=1
                    phi_eta=phi_eta';
                end
                
                vector2=cat(2,0,phi_eta(imin_int:imax_int),0,0);
                GM_variance_int=polyarea(vector1,vector2);
                GM_variance_integral(idp)=GM_variance_int;
            else
                GM_variance_integral(idp)=NaN;
            end

            % Save the N2refmean as new variable: N2b (for background N2)
            N2b(idp)=N2refmean;

            % Compute dissipation rate (epsilon) from the integrated
            % strain variance
            % strain variance normalized to GM76:
            norm_strain_variance(idp)=variance_integral(idp)^2/GM_variance_integral(idp)^2;
            epsilon(idp)=parameters.epsilon0*N2refmean/parameters.N0^2*norm_strain_variance(idp);

            % Modify esimate of epsilon by a few factors (see Kurt's papers
            % for details);  First a correction factor that depends on
            % latitude.  Second a correction factor that depends on the
            % shear to strain ratio.

            extra1 = (fmean/parameters.f0) * acosh(sqrt(N2refmean)/fmean)/acosh(parameters.N0/parameters.f0);
            epsilon(idp) = epsilon(idp)*extra1;

            extra2 = 1/(6*sqrt(2))*(parameters.R*(parameters.R+1))/(sqrt(abs(parameters.R-1)));
            epsilon(idp)=epsilon(idp)*extra2;

            % Finally from epsilon calculate diapycnal diffusivity
            % (kappa) assuming the Osborn relation:
            kappa(idp)=parameters.gamma*epsilon(idp)/N2refmean; 

            % Add variables to float structure:
            mixing_data_gridded.epsilon_strain(idp,i)=epsilon(idp);
            mixing_data_gridded.Kz_strain(idp,i)=kappa(idp);
            mixing_data_gridded.P_m_strain(idp,i)=p_grid(idp);
            mixing_data_gridded.strain_variance(idp,i)=variance_integral(idp);
            mixing_data_gridded.N2b(idp,i)=N2b(idp)'';
            mixing_data_gridded.critiWave_strain(idp,i)=minwavelength(idp);

        end
    end   
end

    % Index down to which mixing value that can be trusted:
    ii_max=max(find(p_grid<(p_max-dp_overlap)));
    mixing_data_gridded.epsilon_strain(ii_max+1:end,i)=NaN;
    mixing_data_gridded.Kz_strain(ii_max+1:end,i)=NaN;
    mixing_data_gridded.P_m_strain(ii_max+1:end,i)=NaN;
    mixing_data_gridded.N2b(ii_max+1:end,i)=NaN;

end

save('mixing_data_gridded.mat','-regexp','^mixing_data_gridded','^flt');