function [mixing_data_gridded] = mx_mixing_analysis(float_data,run,fig,directory,mx_parameters)

% mx_mixing_analysis           Mixing estimates from EM-APEX float profiles
%==========================================================================
%
% USAGE:
%  mixing_data = mx_mixing_analysis(float_data,run,fig,directory,mx_parameters)
%
% DESCRIPTION:
%  This function applies a shear-strain finescale parameterization to 
% vertical profiles from EM-APEX floats and outputs the dissipatino rate
% and diffusivity.
%
% INPUT:
%   float_data  = name of .mat file to be analysed e.g. 'float_data_v2'
%   run         = index of run e.g. 'a'
%   fig         = turns figure display on and off: either 'on' or 'off'.
%                 In both case, figures are saved to drive.  
%   directory   = pwd by default. This is the directory in which the
%                 figures will be saved.
%
% The float_data .mat file has to be a structure array as follow:
%
% F3951 =                           % where 3951 is the float number
%
% 1x65 struct array with fields:    % where 65 is the number of profiles
%
% and where for each profile, the data format is:
%        float_wmoid: '1901142'     % float WMO ID
%              fltid: '4051'
%     profile_number: 1
%        surface_mlt: 7.3374e+05
%                lon: 68.7386
%                lat: -43.7382
%            ctd_mlt: [1x609 double]
%           Pctd_cal: [1x609 double]
%              S_cal: [1x609 double]
%              T_cal: [1x609 double]
%             ef_mlt: [1x464 double]
%            Pef_cal: [1x464 double]
%             U1_abs: [1x464 double]
%             U2_abs: [1x464 double]
%             V1_abs: [1x464 double]
%             V2_abs: [1x464 double]
%
% PARAMETERS:
% The parameters are saved in mx_parameters.mat. Some of them have to be
% changed to reflect the data type, location and resolution.
%
% U='1';              % U velocity sensor: either '1' or '2'
% V='1';              % V velocity sensor: either '1' or '2'
% moving_window=20;
% dzN2ref=24;
% dzN2=6;             % differential length for N2 calculation: e.g. 6 [dbar]
% drho=0.03;          % density gradient to derive the mixed layer depth: 0.03
% dz=3;               % main data set pressure grid interval [m]
% dzs=6;
% fftpt=128;          % Number of points for the fast fourier transform eg
%                       128 but could be 32,64,128... or any power of 2
% lzmin_fixed=50;     % mini wavelength integration for shear/strain spectra	[m]
% lzmax_fixed=300;    % maxi wavelength integration for shear/strain spectra 	[m]
%
% Constants
% R=5;                  % Average shear to strain ratio
% gamma=0.2;            % Mixing efficiency gamma
% epsilon0=8*10^(-10);  % from the GM76 model
% N0=0.00524;           % from the GM76 model
% f0=sw_f(32.5);        % from the GM76 model
% E=6.3e-5;             % dimensionless energy level from the GM76 model
% b=1300;               % scale depth of thermocline [dbar]
% jstar=3;              % mode number
%
% Spectral method: 
% Choice involves trade-offs between confidence and variance perservation 
% spectral_method='Tycho2'; % Tycho2 = 10 sin^2 window
% Decomposition spectral method for both CW and CCW:
% cospectral_method='Tycho2_cospectra';
%
% Spectral corrections:
% Switches for various spectral corrections. Models to correct the high 
% frequency portion of the spectra due to instrument limits.
% switch_fd = 1;  % first-differencing correction when 1 ->correction is on
%
% Wavelength of integral:
% Wavelength integration for shear and strain spectra (finestructure
% epsilon comes from an integral of shear and strain power over a certain
% wavenumber/wavelength range - this sets the limits of wavelength
% integration) ~ to internal waveband.
% lzmin_fixed;     % This value is very sensitive
% lzmax_fixed;     % 300m is quite a typical number
% Threshold to determine lzmin where lzmin is the minimum wavelength for
% which the noise spectra is less critical ratio * spectrum
% crit_rat=NaN;   
% Minimum wavelength threshold - lzmin is set to the maximum of 
% lzmin_threshold and lzmin determinded from the noise threshold
% lzmin_threshold=NaN;                    
%
%
% OUTPUT:
%  mixing_data_gridded.mat  
%
%                fltid: [1x36 double]   Float number 
%       profile_number: [1x36 double]   Profile number for this data
%                  lon: [1x36 double]   Longitude in o
%                  lat: [1x36 double]   Latitude in o
%          surface_mlt: [1x36 double]   Profile surface time
%                  MLD: [1x36 double]   Mixed layer depth m
%                    P: [550x36 double] Pressure
%                   SA: [550x36 double] Absolute salinity in g/kg
%                   CT: [550x36 double] Conservative temperature oC
%              sigma_0: [550x36 double] Potential density ref. to sea surface kg/m^3
%                   N2: [550x36 double] Buyoancy frequency rad/s
%               N2_ref: [550x36 double] Reference buoyancy frequency
%              N2_100m: [550x36 double] Buoyancy frequency over 100m vertical window
%               strain:	[550x36 double]	Strain					
%             fallrate:	[550x36 double]	Fall rate of the EM-APEX float 
%                    U:	[550x36 double]	East horizontal velocity
%                    V:	[550x36 double]	North horizontal velocity
%                speed:	[550x36 double]	Current speed
%                shear:	[550x36 double]	Shear
%        epsilon_shear:	[52x36 double]	Dissipation rate from shear parameterization
%             Kz_shear:	[52x36 double]	Diffusivity from shear parameterization 
%            P_m_shear:	[52x36 double]	Pressure for shear data \si{dbar}
%       shear_variance:	[52x36 double]	Shear variance			
%        CW_S_variance:	[52x36 double]	Clockwise shear variance	
%       CCW_S_variance:	[52x36 double]	Counter-clockwise shear variance
%             Mc_shear:	[52x36 double]	Cutoff vertical wavenumber from shear 
%       epsilon_strain:	[65x36 double]	Dissipation rate from strain parameterization
%            Kz_strain:	[65x36 double]	Diffusivity from strain parameterization
%           P_m_strain:	[65x36 double]	Pressure for strain data \si{dbar}
%      strain_variance:	[65x36 double]	Strain variance	
%     critiWave_strain:	[65x36 double]	Cutoff vertical wavenumber from strain
%              epsilon:	[550x36 double]	Dissipation rate from shear-strain parameterization
%                   Kz:	[550x36 double]	Diffusivity from shear-strain parameterization
%                  P_m:	[550x36 double]	Pressure grid for mixing data
%                   Rw:	[550x36 double]	Shear-to-strain variance ratio  
%
% AUTHOR: 
%  Amelie MEYER
%
% VERSION NUMBER: 1.0 (16th June, 2014)
%
% RERENCE:  A. Meyer, B.M. Sloyan, K.L. Polzin, H.E. Phillips, and N.L. 
%           Bindoff. Mixing variability in the Southern Ocean. Journal of 
%           Physical Oceanography, 45,966-987, 2015.
%==========================================================================

%% Check input parameters are defined
if ~(nargin == 5)
    fprintf(2,'Input variables for mx_mixing_analysis are missing... default options will be used!!\n');
end

% If input variables are not defined, use the default options
if ~exist('directory','var')
    directory=pwd;
elseif ~ischar(directory)
    directory=pwd;
end
if ~exist('fig','var')
    fig='off';
elseif ~ischar(fig)
    fig='off';
end
if ~exist('float_data','var')
    float_data='float_data_vmx';
elseif ~ischar(float_data)
    float_data='float_data_vmx';
end
if ~exist('run','var')
    run='test_run';
elseif ~ischar(run)
    run='test_run';
end
if ~exist('mx_parameters','var')
    mx_parameters='mx_parameters';
elseif ~ischar(mx_parameters)
    mx_parameters='mx_parameters';
end

%% Load and build datasets
load(mx_parameters);
mx_build_initial_data(float_data)

%% Derive fallrate of the EM-APEX floats
mx_derive_fallrate

%% Grid CTD data onto 2.2dbar and EM on 3dbar
mx_grid_initialdata(parameters.U,parameters.V)

%% Derive initial variables
mx_derive_abs_T_S
mx_derive_potential_density_anomaly
mx_derive_N2(parameters.dzN2)
mx_derive_mixed_layer_depth(parameters.drho)
mx_derive_current_speed

%% Plot main variables
mx_grid_all_initial_data

mx_plot_temperature(fig,directory)
mx_plot_salinity(fig,directory)
mx_plot_N2(fig,directory)
mx_plot_mixed_layer_depth(fig,directory)
mx_plot_current_speed(fig,directory)

%% Derive mixing variables
mx_derive_N2_ref(parameters.moving_window,parameters.dzN2ref)
mx_derive_N2_100m
mx_derive_strain
mx_derive_shear(parameters.dz,parameters.dzs)

%% Derive mixing
mx_grid_mixingdata(parameters.dz)
mixing_data_gridded=mx_derive_mixing(parameters.dz,parameters.fftpt,parameters.lzmin_fixed,parameters.lzmax_fixed,mx_parameters,run);

%% Plot main mixing variables
mx_plot_dissipation_rate(fig,directory,run)
mx_plot_diffusivity(fig,directory,run)
mx_plot_CCW_CW(fig,directory,run)
mx_plot_Rw(fig,directory,run)
