function [nn,delnn]=nsq_modbs(s,t,p,dz,lat,varargin)

%======================================================================
% NSQ_MODBS        
%
% [nn,delnn]=nsq_modbs(s,t,p,dz,lat,varargin)
%
% DESCRIPTION:
%... Calculates squared Brunt-Viasala frequecy (N^2),
%... units [rads/sec]^2, using a linear regression over the pressure window
%... specified by input dz to calculate the sigma-theta(sub)z.  
%... This function runs fairly slow, as it must recompute the potential
%... density at each "bottle" using the appropriate reference pressure.
%... Potential Density (reference pressure is the middle pressure of the 
%... of the fitting window for which N^2 is calculated) is calculated
%... from the salinity, temperature and pressure. A linear regression 
%... bewteen the density and pressure is calculated using the 
%... form Ax = b, where is x = A\b over a window 2*dz is obtained. 2nd
%... polynominal regression are used at the top and bottom of profile.
%... N^2 is then calculated  
%...            N^2 = -(g/pdens)*x  [s^-2] middle profile
%
% INPUT:
%    s   = salinity (vertical profile)
%    t   = temperature (vertical profile)
%    p   = pressure (vertical profile)
%    dz  = differential length (1 value)
%    lat = latitude for each vertical profile (1 value)
%
% OUTPUT:
%    nn    = Brunt Vaisala Frequency in rad/s^2
%    delnn = Error on slope?
%              
% AUTHOR:  Based on Kurt original nsq_mod.m code
%=======================================================================

% @(#)nsq_modbs.m  Revision:     18/05/2009   Amelie Meyer  
%=======================================================================

[M,N]=size(s);

%% .. setup indice over which to calculate N^2
if nargin == 6          % Nargin is the number of arguments in the function
  mld = varargin{:};
    for j = 1:N
      f_ind(j) = length(find(p(:,j) < mld));
      [l_val,l_ind] = lastgood(p);
    end

elseif nargin == 5    % Thats if we only put in s t p dz and lat
    [f_val,f_ind] = firstgood(p); 
    [l_val,l_ind] = lastgood(p);
end

if length(p)~=M,        % Checks that pressure and S are same matrix size
error('size of p does not match s,t')
end;

%% ... Calculate depth in meters (0 at the surface and towards -inf at depth)
z=-sw_dpth(p,lat);      % Seawater library

%% ... define the error
delr=(5e-4)/sqrt(9);

%%
mdz=nanmean(diff(p));
interval=abs(round(mean(mdz)));      % Need the absolute value of the step because it was negative when float on way up...
r=dz/interval; % Original script
%r=round(dz/interval); % need to make sure that r is an integer...

%% .. Calculate the error on the slope
F=[ones(2*r+1,1),(1:0.5:r+1)'];
P=(delr^2)*inv(F'*F);
delnn=sqrt(P(2,2));

%% Calculate N2
nn=nan*ones(M,N);
%%%%% do a loop to compute linear regressions, 1 column at a time
for j=1:N 
mmf = f_ind(j);
mml = l_ind(j);

 if mml < r*4+1
     nn(:,j) = NaN;
 else

  % interpolate temperature and salinity if there is a nan in the station
     if isfinite(find(isnan(s(mmf:mml,j))))
       tt = find(isnan(s(mmf:mml,j)));
       ind = (mmf:mml); ind(tt) = [];
       s(mmf:mml,j) = interp1(p(ind,j),s(ind,j),p(mmf:mml,j));
       t(mmf:mml,j) = interp1(p(ind,j),t(ind,j),p(mmf:mml,j));
     end
     clear ind

%    Middle first
    for k=(r+mmf):(mml-r)
      E=[ones((2*r+1),1),z(k-r:k+r,j)];
      pd=sw_pden(s(k-r:k+r,j),t(k-r:k+r,j),p(k-r:k+r,j),p(k,j));  
      A=E\pd;
        nn(k,j)=-(sw_g(lat(j),z(k,j))/pd(ceil(length(pd)/2)))*A(2);
    end;

%    Bottom extrapolation next
    k=mml-r; 
     E=[ones((2*r+1),1),z(mml-2*r:mml,j),z(mml-2*r:mml,j).^2];
     pd=sw_pden(s(mml-2*r:mml,j),t(mml-2*r:mml,j),p(mml-2*r:mml,j),p(mml-r,j));
      A=E\pd;
         nn(mml-r+1:mml,j)=-(sw_g(lat(j),z(k,j))/pd(r+1))*(A(2)+2*A(3)*z(mml-r+1:mml,j));

%    Top last
    k=r+mmf;
     E=[ones((2*r+1),1),z(mmf:2*r+mmf,j),z(mmf:2*r+mmf,j).^2,z(mmf:2*r+mmf,j).^3];
     pd=sw_pden(s(mmf:2*r+mmf,j),t(mmf:2*r+mmf,j),p(mmf:2*r+mmf,j),p(r+mmf,j));
      A=E\pd;
          nn(mmf:k,j)=-(sw_g(lat(j),z(k,j))/pd(r+1))*(A(2)+2*A(3)*z(mmf:k,j)+3*A(4)*z(mmf:k,j).^2);

%   Try to meld the two estimates together gracefully
	nn_meld(r+mmf:2*r+mmf)=-(sw_g(lat(j),z(k,j))/pd(r+1))*(A(2)+2*A(3)*z(r+mmf:2*r+mmf,j)+3*A(4)*z(r+mmf:2*r+mmf,j).^2);
    for m=1:r
	nn(r+mmf+m,j)=(m/r)*nn(r+mmf+m,j)+(1-(m/r))*nn_meld(r+mmf+m);
    end;
 end;
end;

%% NOTES

% Problem with dz: needs to be a mutiple of diff(p) otherwise r isn't an
% integer and the function crashes. - Have to round r or always put dZ aa
% a multiple of resolution

% Do I go for dz=3  or dz=60?? depends... probably do both.. actually
% smallest I can go for is twice the resolution so min dz=6 and up to dz=90
% is a good averaging...

% Why is delnn 0 or infinity??? bernadette is not sure
 